/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 * @version: Release 1.0 $date:  March 1, 1999$
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 */

package xset;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import java.rmi.*;
import java.rmi.server.*;

import xset.XSetServiceIF;

/**
 * Main applet class to demonstrate a primitive UI to XSet
 * For more info on XSet, see doc/XSetReadMe.html or
 * http://www.cs.berkeley.edu/~ravenben/ninja/xset.html
 */

public class XSetApplet extends Applet implements ItemListener, ActionListener
{
  XSetServiceIF XSetService;
  String hostname = null;
  String path = null;
  String query = "<?xml version=\"1.0\"?><PRINTCAP><COLOR>YES</COLOR></PRINTCAP>";
  TextArea resultWindow;
  TextArea queryWindow;
  String out;
  Vector results;
  Hashtable attributes;
  Vector queryItems;

  private Panel genQuery, qType, attr, attrNamePanel, attrValuePanel;
  private Panel attrBottomPanel, rangeQuery, queryButton, actionPanel;
  private Panel tagNamePanel, tagValuePanel, lastQueryPanel;
  private CheckboxGroup queryType;
  private TextField attrName, attrValue, rqleftValue, rqrightValue;
  private TextField tagValue, tagName, pathField;
  private Button addAttrButton, clearAttrButton, qButton, clearQButton;
  private Button doButton, addFilesButton, clearXSetButton, addQueryItem;
  private Choice rqleftOper,  rqrightOper, rqType;
  private Checkbox Range, Value;

  public void init() {
    String [] servlist;
    java.util.Enumeration e;

    hostname = getParameter("HOSTNAME");
    path = getParameter("XMLPATH");
    attributes = new Hashtable();
    queryItems = new Vector();

    setLayout(new FlowLayout(FlowLayout.CENTER,20,5));
    setBackground(new Color(246,246,225));
    setForeground(new Color(0, 0, 255));

    actionPanel = new Panel();
      Panel mini = new Panel(new BorderLayout());
        clearXSetButton = new Button("Clear XSet");
	clearXSetButton.addActionListener(this);
	addFilesButton = new Button("Add Files");
	addFilesButton.addActionListener(this);
      mini.add(clearXSetButton, "North");
      mini.add(addFilesButton, "South");
      actionPanel.add(mini);
      pathField = new TextField(path, 80);
      pathField.addActionListener(this);
      actionPanel.add(pathField);
    add(actionPanel);

    genQuery = new Panel(new BorderLayout());
      qType = new Panel(new BorderLayout(5,0));
        queryType = new CheckboxGroup();
	qType.add(new Label("Query Type", Label.CENTER), "North");
	Range = new Checkbox("Range", queryType, false);
	Value = new Checkbox("Value", queryType, true);
	qType.add(Range, "West");
	qType.add(Value, "East");
      genQuery.add(qType, "North");

      attr = new Panel(new BorderLayout(10,0));
        attr.add(new Label("Attributes", Label.CENTER), "North");
	attrNamePanel = new Panel(new BorderLayout());
	  attrNamePanel.add(new Label("Name", Label.CENTER), "North");
	  attrName = new TextField(15);
	  attrNamePanel.add(attrName, "South");
	attrValuePanel = new Panel(new BorderLayout());
	  attrValuePanel.add(new Label("Value", Label.CENTER), "North");
	  attrValue = new TextField(15);
	  attrValuePanel.add(attrValue, "South");
	attr.add(attrNamePanel, "West");
	attr.add(attrValuePanel, "East");
	attrBottomPanel = new Panel(new FlowLayout());
	  addAttrButton = new Button("Add/Replace Attribute");
	  addAttrButton.addActionListener(this);
	  clearAttrButton = new Button("Clear Attributes");
	  clearAttrButton.addActionListener(this);
	attrBottomPanel.add(addAttrButton);
	attrBottomPanel.add(clearAttrButton);
	attr.add(attrBottomPanel, "South");
      genQuery.add(attr, "Center");
      
      rangeQuery = new Panel(new BorderLayout());
        rangeQuery.add(new Label("Range Query", Label.CENTER), "North");

	Panel rqleft = new Panel(new BorderLayout());
	  rqleftOper = new Choice();
	  rqleftOper.add("None"); rqleftOper.add(">"); rqleftOper.add(">=");
          rqleft.add(rqleftOper, "North");
	  rqleftValue = new TextField(10);
	  rqleft.add(rqleftValue, "South");
	rangeQuery.add(rqleft, "West");

	rqType = new Choice();
	rqType.add("STRING"); rqType.add("INTEGER"); rqType.add("FLOAT");
	rangeQuery.add(rqType, "Center");

	Panel rqright = new Panel(new BorderLayout());
	  rqrightOper = new Choice();
	  rqrightOper.add("None"); rqrightOper.add("<"); rqrightOper.add("<=");
          rqright.add(rqrightOper, "North");
	  rqrightValue = new TextField(10);
	  rqright.add(rqrightValue, "South");
	rangeQuery.add(rqright, "East");

	queryButton = new Panel(new BorderLayout());
	  qButton = new Button("Generate Query");
	  qButton.addActionListener(this);
	queryButton.add(qButton, "North");
	tagNamePanel = new Panel(new BorderLayout());
	  tagNamePanel.add(new Label("Tag Name", Label.CENTER), "North");
	  tagName = new TextField(10);
	  tagNamePanel.add(tagName, "South");
	tagValuePanel = new Panel(new BorderLayout());
	  tagValuePanel.add(new Label("Tag Value", Label.CENTER), "North");
	  tagValue = new TextField(10);
	  tagValuePanel.add(tagValue, "South");
	queryButton.add(tagNamePanel, "West");
	queryButton.add(tagValuePanel, "East");
	lastQueryPanel = new Panel();
	  addQueryItem = new Button("Add QueryItem");
	  addQueryItem.addActionListener(this);
	lastQueryPanel.add(addQueryItem);
	  clearQButton = new Button("Clear QueryItems");
	  clearQButton.addActionListener(this);
	lastQueryPanel.add(clearQButton);
	  doButton = new Button("Process Query");
	  doButton.addActionListener(this);
	lastQueryPanel.add(doButton);
	queryButton.add(lastQueryPanel, "South");
	rangeQuery.add(queryButton, "South");
      genQuery.add(rangeQuery, "South");
    add(genQuery);

    queryWindow = new TextArea(15, 70);
    queryWindow.setEditable(true);
    add(queryWindow);
    resultWindow = new TextArea("", 30, 120,TextArea.SCROLLBARS_VERTICAL_ONLY);
    resultWindow.setEditable(false);
    add(resultWindow);

    String url = "rmi:///";
    try {
      out = "XSet Demo Applet\n";
      XSetService = (XSetServiceIF) Naming.lookup(url+"XSet");
      if (XSetService == null) {
	out = out+"UTOH\n";
	return;
      }
      XSetService.XSetAddFiles(path);
      results = XSetService.XSetQuery(query);
      if (results != null) {
	e = results.elements();
	int i = 0;
	while(e.hasMoreElements()) {
	  out = out+"Match:\n"+(String) e.nextElement()+"\n";
	}
      } else out = out+"NO MATCH\n";
      /*
      XSetService.XSetClearAll();
      results = XSetService.XSetQuery(query);
      if (results != null) {
	e = results.elements();
	int i = 0;
	while(e.hasMoreElements()) {
	  out = out+"Match:\n"+(String) e.nextElement()+"\n";
	}
      } else out=out+"NO MATCH\n";
      */
    } catch (Exception ex) {
      out=out+"Exception while execution: "+ex+"\n";
      return;
    }
  }

  public void start() {
  }

  public void itemStateChanged(ItemEvent ie) {

  }

  public void actionPerformed(ActionEvent ae) {
    String arg = ae.getActionCommand();
    if (arg.equals("Clear XSet")) {
      try {
	XSetService.XSetClearAll();
      } catch (java.rmi.RemoteException re) {
	re.printStackTrace();
      }
      out = "Clear Set successful";
      repaint();
    } else if (arg.equals("Add Files")) {
      try {
	XSetService.XSetAddFiles(path);
      } catch (java.rmi.RemoteException re) {
	re.printStackTrace();
      }
      out = "Add files successful";
      repaint();
    } else if (arg.equals("Add/Replace Attribute")) {
      String attributeName = attrName.getText();
      String attributeValue = attrValue.getText();
      if (attributes.containsKey(attributeName)) {
	attributes.remove(attributeName);
      }
      attributes.put(attributeName, attributeValue);
      out = "Attribute "+attributeName+" updated with value: " +
	attributeValue;
      repaint();
    } else if (arg.equals("Clear Attributes")) {
      attributes = new Hashtable();
      out = "All attributes cleared";
      repaint();
    } else if (arg.equals("Generate Query")) {
      query = "<?xml version=\"1.0\"?>\n<PRINTCAP>";
      Enumeration tempE = queryItems.elements();
      while(tempE.hasMoreElements()) {
	query=query+"\n"+(String) tempE.nextElement();
      }
      query = query+"\n</PRINTCAP>";
      repaint();
    } else if (arg.equals("Clear QueryItems")) {
      queryItems = new Vector();
      out = "Query items cleared";
      repaint();
    } else if (arg.equals("Add QueryItem")) {
      String tempQuery = "<"+tagName.getText();
      Enumeration tempE = attributes.keys();
      String tempKey;
      while (tempE.hasMoreElements()) {
	tempKey = (String) tempE.nextElement();
	tempQuery= tempQuery+" "+tempKey+"=\""+(String)attributes.get(tempKey);
	tempQuery = tempQuery+"\"";
      }
      if (queryType.getSelectedCheckbox() == Range) {
	if (!rqleftValue.equals("None")) {
	  String rangeleftOper = rqleftOper.getSelectedItem();
	  if (rangeleftOper.equals(">")) {
	    rangeleftOper = "MORE_THAN";
	  } else if (rangeleftOper.equals(">=")) {
	    rangeleftOper = "MORE_EQUAL";
	  } else rangeleftOper = null;
	  if (rangeleftOper != null) {
	    tempQuery = tempQuery+" "+rangeleftOper+"=\""+
	      rqleftValue.getText()+"\"";
	  }
	} 
	if (!rqrightValue.equals("None")) {
	  String rangerightOper = rqrightOper.getSelectedItem();
	  if (rangerightOper.equals("<")) {
	    rangerightOper = "LESS_THAN";
	  } else if (rangerightOper.equals("<=")) {
	    rangerightOper = "LESS_EQUAL";
	  } else rangerightOper = null;
	  if (rangerightOper != null) {
	    tempQuery = tempQuery+" "+rangerightOper+"=\""+
	      rqrightValue.getText()+"\"";
	  }
	  tempQuery = tempQuery+" KEY_TYPE=\""+rqType.getSelectedItem()+"\"";
	  tempQuery = tempQuery+">RANGEQUERY</"+tagName.getText()+">";
	}
      } else {
	tempQuery=tempQuery+">"+tagValue.getText()+"</"+tagName.getText()+">";
      }
      queryItems.addElement(tempQuery);
      out = "Query Item created:\n"+tempQuery;
      repaint();
    } else if (arg.equals("Process Query")) {
      results = null;
      try {
	results = XSetService.XSetQuery(query);
      } catch (java.rmi.RemoteException re) {
	re.printStackTrace();
      }
      out = "Query processed:\n"+query+"\n\n";
      if (results != null) {
	Enumeration resultsE = results.elements();
	while(resultsE.hasMoreElements()) {
	  out = out+"Match:\n"+(String) resultsE.nextElement()+"\n";
	}
      } else out = out+"No matches found!";
      repaint();
    } else { // not a button, something else
      Object source = ae.getSource();
      if (source == pathField) {
	path = pathField.getText();
      }
    }
  }

  public void paint(Graphics g) {
    resultWindow.setText(out);
    queryWindow.setText("Query:\n"+query);
  }

  public void stop() {

  }
}
