/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 * @version: Release 1.0 $date:  March 1, 1999$
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 */

package xset;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.RemoteException;
import com.ms.xml.om.*;
import com.ms.xml.util.*;

public class XSetDBTest {
  private SETserver server;
  private String XSetName;
  private Runtime run;

  private int filesindexed;
  private long indextime;
  private long querytime;
  private int memoryleft;
  private int memorytotal;
  String query1 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>lvl-sun669.usc.edu</SOURCEIP></WEBLOG>";
  String query2 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>crawl3.atext.com</SOURCEIP></WEBLOG>";
  String query3 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>intergate.sbhsd.k12.ca.us</SOURCEIP></WEBLOG>";
  String query4 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>p18-term1-and.netdirect.net</SOURCEIP></WEBLOG>";
  String query5 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>209.101.58.234</SOURCEIP></WEBLOG>";
  String query6 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>novartis2-bh.us.novartis.com</SOURCEIP></WEBLOG>";
  String query7 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>kerkyra.epsilon.com.gr</SOURCEIP></WEBLOG>";
  String query8 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>lsi-station1.lsi.dsc.ufpb.br</SOURCEIP></WEBLOG>";
  String query9 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>ckc-114-60.reshall.berkeley.edu</SOURCEIP></WEBLOG>";
  String query10 = "<?xml version=\"1.0\"?><WEBLOG><SOURCEIP>205.238.147.244</SOURCEIP></WEBLOG>";

  public XSetDBTest() {
  }

  public static void main(String [] argv) {
    XSetDBTest xs = new XSetDBTest();
    xs.run();
  }
  
  public void run() {
    XSetName = "set";
    System.out.println("XSetService: started.");
    System.out.println("   XSetName: "+XSetName);
    int load = 20;
    filesindexed = 0;
    run = Runtime.getRuntime();
    try {
      XSetClearAll();
      for (int i =0; i<load; i++) {
	  XSetAddFiles("/usr/home/research/xset/db/"+i);
      }
      /*      long time;
      Vector ans = null;
      System.out.println("Database loaded with "+load+" directories\n");
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
        ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("1 Xset query: "+time);
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
	ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("2nd Xset query: "+time);
      if (ans == null) return;
      Enumeration e = ans.elements();
      while (e.hasMoreElements()) {
	  System.out.println("Result: >>"+
			     ((ServiceEntry)e.nextElement()).getXML());
      }
      */
     } catch (Exception ex) {ex.printStackTrace();}
  }

  public void XSetAddFiles(String path) {
    //    System.out.println("XSetAddFiles request received!");
    File dir = new File(path);
    String [] xmlFiles = null;
    Document d = null;

    if ((xmlFiles = dir.list(new ExtensionFilter("xml"))) == null) {
      System.out.println("Directory not found or inaccessible!");
      return;
    }
    int i = 0;
    try {
      for (i = 0; i < xmlFiles.length; i++) {
	//	System.out.println("File: "+xmlFiles[i]+ " loaded.");
	d = new Document();
	d.load(SETutils.parseFile(path+"/"+xmlFiles[i]));
/*	if (i==0) indextime = System.currentTimeMillis();
	if ((i+1) % 100 == 0) {
	    indextime = System.currentTimeMillis() - indextime;
	    System.out.println("Documents: "+(filesindexed+1)+" 100-Index: "+
			       indextime);
	    indextime = System.currentTimeMillis();
	    server.SETregister(d, XSetName);
	} else */
	    server.SETregister(d, XSetName);
	filesindexed++;

	if ((i+1) % 100 == 0) {
// 	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
// 			   run.totalMemory()+"\n");
 	run.gc();
 	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
			   run.totalMemory()+"\n");
	    Vector ans = null;
	    //run query latency tests
	    querytime = System.currentTimeMillis();
	    for (int j = 0; j<10; j++) {
		ans = XSetQuery(query1);
		ans = XSetQuery(query2);
		ans = XSetQuery(query3);
		ans = XSetQuery(query4);
		ans = XSetQuery(query5);
		ans = XSetQuery(query6);
		ans = XSetQuery(query7);
		ans = XSetQuery(query8);
		ans = XSetQuery(query9);
		ans = XSetQuery(query10);
	    }
	    querytime = System.currentTimeMillis() - querytime;
	    System.out.println("Documents: "+filesindexed+ " Query1: "+
			       querytime);
	    System.out.println("Returned "+ans.size());
	}
	/*	System.out.println(xmlFiles[i]+" indexed.\n");
 	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
 			   run.totalMemory()+"\n");
 	run.gc();
 	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
			   run.totalMemory()+"\n");
	*/
      }
    } catch (FileNotFoundException fnfe) {
      System.err.println("Query file not found!");
      return;
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Parsing ERROR on: "+xmlFiles[i]);
    }
  }

  public void XSetClearAll() {
    System.out.println("XSetClearAll request received!");
    server = new SETserver();
    System.out.println("XSet "+XSetName+ " cleared.");
  }

  public Vector XSetQuery(String xmlQuery) {
    //    System.out.println("XSetQuery request received!");
    Document d = new Document();
    Vector results = null;
    InputStream is = new ByteArrayInputStream(xmlQuery.getBytes());
    try {
      d.load(is);
      results = server.querySET(d, XSetName);
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Error parsing query string: \n"+xmlQuery);
    }
    return results;
  }
}
