/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 * @version: Release 1.0 $date:  March 1, 1999$
 * (c) 1999 by Ben Yanbin Zhao and the Regents of the University of California
 */

package xset;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.rmi.RemoteException;
import com.ms.xml.om.*;
import com.ms.xml.util.*;

public class XSetProfile {
  private SETserver server;
  private String XSetName;
  Runtime run;

  public XSetProfile() {
  }

  public static void main(String [] argv) {
    XSetProfile xs = new XSetProfile();
    xs.run();
  }
  
  public void run() {
    run = Runtime.getRuntime();
    XSetName = "set";
    System.out.println("XSetService: started.");
    System.out.println("   XSetName: "+XSetName);
    try {
      XSetClearAll();
      XSetAddFiles("/usr/home/ravenben/ninja/classpath/ninja/xset/printcaps");
      String query = "<?xml version=\"1.0\"?><PRINTCAP><COLOR>NO</COLOR></PRINTCAP>";
      long time;
      Vector ans = null;
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
        ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("1 Xset query with 1 constraint: "+time);
      query = "<?xml version=\"1.0\"?><PRINTCAP><DUPLEX>YES</DUPLEX><COLOR>NO</COLOR></PRINTCAP>";
      time = System.currentTimeMillis();
      for (int i =0; i<100; i++) {
	ans = XSetQuery(query);
      }
      time = System.currentTimeMillis() - time;
      System.out.println("1 Xset query with 2 constraints: "+time);
      if (ans == null) return;
      Enumeration e = ans.elements();
      while (e.hasMoreElements()) {
	  System.out.println("Result: >>"+
			     ((ServiceEntry)e.nextElement()).getXML());
      }
     } catch (Exception ex) {ex.printStackTrace();}
  }

  public void XSetAddFiles(String path) {
    //    System.out.println("XSetAddFiles request received!");
    File dir = new File(path);
    String [] xmlFiles = null;
    Document d = null;

    if ((xmlFiles = dir.list(new ExtensionFilter("xml"))) == null) {
      System.out.println("Directory not found or inaccessible!");
      return;
    }
    int i = 0;
    try {
      for (i = 0; i < xmlFiles.length; i++) {
	//	System.out.println("File: "+xmlFiles[i]+ " loaded.");
	d = new Document();
	d.load(SETutils.parseFile(path+"/"+xmlFiles[i]));
	server.SETregister(d, XSetName);
	System.out.println(xmlFiles[i]+" indexed.\n");
	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
			   run.totalMemory()+"\n");
	run.gc();
	System.out.println("FreeMemory: "+run.freeMemory()+ "totalMemory: "+
			   run.totalMemory()+"\n");
      }
    } catch (FileNotFoundException fnfe) {
      System.err.println("Query file not found!");
      return;
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Parsing ERROR on: "+xmlFiles[i]);
    }
  }

  public void XSetClearAll() {
    System.out.println("XSetClearAll request received!");
    server = new SETserver();
    System.out.println("XSet "+XSetName+ " cleared.");
  }

  public Vector XSetQuery(String xmlQuery) {
    //    System.out.println("XSetQuery request received!");
    Document d = new Document();
    Vector results = null;
    InputStream is = new ByteArrayInputStream(xmlQuery.getBytes());
    try {
      d.load(is);
      results = server.querySET(d, XSetName);
    } catch (com.ms.xml.parser.ParseException pe) {
      System.err.println("Error parsing query string: \n"+xmlQuery);
    }
    return results;
  }
}
